package jp.kwebs;

import java.util.List;
import javax.json.Json;
import javax.json.JsonObjectBuilder;
import javax.transaction.Transactional;
import javax.ws.rs.Consumes;
import javax.ws.rs.DELETE;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.PUT;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.WebApplicationException;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;
import javax.ws.rs.ext.ExceptionMapper;
import javax.ws.rs.ext.Provider;
import javax.ws.rs.QueryParam;
import javax.ws.rs.DefaultValue;
import io.quarkus.hibernate.orm.panache.PanacheQuery;
import io.quarkus.panache.common.Page;
import io.quarkus.panache.common.Sort;


@Produces(MediaType.APPLICATION_JSON)
@Path("/books")
public class BookResource {
	@GET
	@Path("/{id}")
	public Book getBookById(@PathParam("id") Long id) {
		return	Book.findById(id);	// idで検索する	
	}
	@GET
	public List<Book> getAllBooks() {
		
		return Book.findAll(Sort.by("id")).list();	// 全てのエンティティをidでソートして得る		
	}

	// 検索式による検索、ソート、ページング
	@GET
	@Path("/query")
	public List<Book> getBooks(
			@DefaultValue("")		@QueryParam("query") String query,
			@DefaultValue("price")  @QueryParam("sort") String sort,
			@DefaultValue("0")	  	@QueryParam("index") int index,
			@DefaultValue("3")	  	@QueryParam("size") int size )  {
		
		PanacheQuery<Book> books = Book.find(query, Sort.by(sort));
		return books.page(Page.of(index, size)).list();
	}
	// 検索結果の件数
	@GET
	@Path("/count")
	public long getCount( @DefaultValue("") @QueryParam("query") String query)  {
		return Book.find(query).count();
	}
	// 検索結果のページ数
	@GET
	@Path("/size")
	public int getSize( 
			@DefaultValue("")	@QueryParam("query") String query,	  // 検索式
			@DefaultValue("3")	@QueryParam("size") int size     ) { // ページサイズ
		
		return Book.find(query).page(Page.ofSize(size)).pageCount();
	}
	
	@POST
	@Transactional
	@Consumes(MediaType.APPLICATION_JSON)
	public Response create(Book book) { // 新規追加
		if(Book.findById(book.id)!=null) {
			throw new WebApplicationException(
				"id="+book.id+"のデータはすでにあります", 409);
		}
		book.persist();
		return 	Response.status(201).build();
	}
	
	@PUT
	@Transactional
	@Consumes(MediaType.APPLICATION_JSON)
	public Response update(Book book) { // 更新
		Book bk = Book.findById(book.id);
		if(bk==null) {
			throw new WebApplicationException(
					"id="+book.id+"のデータはありません", 404);
		}
		bk.title = book.title;
		bk.genre = book.genre;
		bk.author = book.author;
		bk.price = book.price;
		bk.stock = book.stock;
		return  Response.ok().build();
	}
	
	@DELETE
	@Transactional
	@Path("/delete/{id}")
	public Response delete(@PathParam("id") Long id) {	// 削除
		Book book = Book.findById(id);
		if(book==null) {
			throw new WebApplicationException(
						"id="+id+"のデータはありません", 404);
		}
		book.delete();
		return Response.noContent().build();
	}
	
	
	@Provider
	public static class ErrorMapper implements ExceptionMapper<Exception> {
		@Override
		public Response toResponse(Exception exception) {
			int code = 500;
			if (exception instanceof WebApplicationException) {
				code = ((WebApplicationException) exception)
									.getResponse().getStatus();
			}

			JsonObjectBuilder json = Json.createObjectBuilder()
				.add("type", exception.getClass().getName())
				.add("code", code);

			if (exception.getMessage() != null) {
				json.add("message", exception.getMessage());			
			}

			return Response.status(code).entity(json.build()).build();
		}
	}

}
